# cody.py (Coin M0 MacroPad example)
# Copyright: 2024 Bradan Lane STUDIO
#
# Open Source License: MIT

'''
This CircuitPython program implements a sample macropad and exercises all of the hardware on the Coin M0.

This is one of two exampes: this is a "Big Mute Button" example.
It treats all touch pads as one button and pressing it toggles mute on / mute off.


Implementation Notes
--------------------
**Hardware:**
    * a Coin M0 with the SAMD21 microcontroller, touch sensors, LEDs, and a speaker

**Software and Dependencies:**
    * CircuitPython firmware (UF2): https://circuitpython.org

**Open Source Repository:**
    * GitLab: https://gitlab.com/bradanlane/coin_m0
    
'''

import board
import time
import digitalio
import touchio
import array

from audiocore import RawSample
from audioio import AudioOut

import usb_hid
from adafruit_hid.keyboard import Keyboard
from adafruit_hid.keycode import Keycode


''' -------------------------------------------------------------------------
    some Coin M0 have three sets of three discreet LEDs
    some Coin M0 have three sets of three Neopixels
------------------------------------------------------------------------- '''
LED_PINS: list = [board.D12, board.D11, board.D10]  # red, green, blue
LED_COUNT:int = 3
leds = [None for i in range(LED_COUNT)]	# start with a list of `None` data
for i in range(LED_COUNT):
    leds[i] = digitalio.DigitalInOut(LED_PINS[i])
    leds[i].direction = digitalio.Direction.OUTPUT
    leds[i].value = False # the initial status LED is OFF


def leds_on():
    for i in range(LED_COUNT):
        leds[i].value = True
        
def leds_off():
    for i in range(LED_COUNT):
        leds[i].value = False

''' -------------------------------------------------------------------------
    the Coin M0 has three touch sensors
------------------------------------------------------------------------- '''
TOUCH_PINS: list = [board.TOUCH1, board.TOUCH2, board.TOUCH3]
TOUCH_COUNT:int = 3
touch_sensors = [None for i in range(TOUCH_COUNT)]	# start with a list of `None` data
touching = [0] * TOUCH_COUNT

for i in range(TOUCH_COUNT):
    touch_sensors[i] = touchio.TouchIn(TOUCH_PINS[i])
    # the minimum sensitivity (aka threshold) is set when the touch sensor is created
    #print(f"initialized touch {i:d} with threshold {touch_sensors[i].threshold:d}")


''' -------------------------------------------------------------------------
    the Coin M0 has a speaker which is a true analog device
------------------------------------------------------------------------- '''
SAMPLERATE = 8000
music_scale:list = [523.25, 587.33, 659.33, 698.46, 783.99, 880.00, 987.77, 1046.50, 1174.66]
audio = AudioOut(board.SPEAKER)


''' -------------------------------------------------------------------------
    the Coin M0 has a USB HID and may be used as a keyboard or MIDI device
    Ref: https://docs.circuitpython.org/projects/hid/en/latest/api.html
------------------------------------------------------------------------- '''
kbd = Keyboard(usb_hid.devices)
# to use a single key, lookup the Keycode in the linked webpage above
# to send a key sequence, use a tuple of Keycodes, A tuple is a comma list within parentheses
#    eg: (Keycode.CONTROL, Keycode.LEFT_SHIFT, Keycode.KEYPAD_FIVE)

# the MuteKey App default shortcut is SHIFT+COMMAND+W
mute = (Keycode.SHIFT, Keycode.COMMAND, Keycode.W)

def send_keys(command=None):
    if command == None:
        return False

    if isinstance(command, str):
        kbd.write(command)              # output a text string
    elif isinstance(command, int):
        kbd.send(command)               # output a keycode
    elif isinstance(command, list):
        for cmd in command:             # loop thru commands in list
            send_keys(cmd)              # recurse to output each command as a separate key
    elif isinstance(command, tuple):
        for cmd in command:             # output tuple by pressing all keys together
            kbd.press(cmd)              # press each key (best for key combinations)
    else: # unknown
        # this is a debugging print message, only visible with a serial monitor
        print("Unknown key command: ", command)
        return False
    kbd.release_all()
    return True

    
''' -------------------------------------------------------------------------
    Now that the hardware is initialized, we have a few utility functions
    to help us interact with the hardware
------------------------------------------------------------------------- '''
def tone_on(frequency:float=440.0):
    #print(f"Tone On ({frequency:f})")
    frequency = int(frequency)
    half_length = int(SAMPLERATE / frequency / 2)
    wave = array.array("H", [0] * (half_length * 2))
    for i in range(half_length):
        wave[i] = 32768
        wave[i + half_length] = 0
    audio.play(RawSample(wave), loop=True)

def tone_off():
    #print("Tone Off")
    audio.stop()

def tone(frequency:float=440.0, duration:int=1.0):
    ''' produce the given sound for the given amount of time '''
    tone_on(frequency)
    time.sleep(duration)
    tone_off()



''' -------------------------------------------------------------------------
    The main loop of the program:
    Here we will check the status of the touch sensors and respond
    by turning on the corresponding LED, playing a tone, and sending
    a keycode to the computer.
------------------------------------------------------------------------- '''

active = False      # the the mute function is active
QUIET = True        # turn off the speaker

pressed = False
prior = False
try:
    while True:

        prior = pressed
        pressed = False

        # check if any pads are touched        
        for i in range(TOUCH_COUNT):
            pressed = pressed or touch_sensors[i].value

        if pressed and prior:   # still pressed
            continue
        if not prior and pressed:   # just pressed
            # either turn on or off the mute function
            send_keys(mute)
            
            if active:
                active = False
                leds_off()
                if not QUIET:
                    tone_off()
            else:
                active = True
                leds_on()
                if not QUIET:
                    tone_on(music_scale[0])

        time.sleep(0.10)

finally:
    # any cleanup code goes here
    tone_off()
    for i in range(LED_COUNT):
        leds[i].value = False
    # turn off Neopixels if present
    print("Goodbye!")
    time.sleep(0.5)
    pass

